@@ -4,9 +4,7 @@
  * GChemPaint library
  * application.h 
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -31,11 +29,20 @@
 #include "bonobo/bonobo-ui-node.h"
 #include <gcu/application.h>
 #include <gcu/dialog.h>
+#include <set>
 #include <string>
 #include <map>
 
 using namespace gcu;
 
+typedef struct
+{
+	char const *name;
+	unsigned char const *data_24;
+} gcpIconDesc;
+
+class gcpWindow;
+	
 class gcpApplication: public Application
 {
 public:
@@ -50,11 +57,11 @@ public:
 	virtual void SetStatusText(const char* text);
 	void AddUI(const char* filename);
 	virtual GtkWindow* GetWindow() = 0;
-	virtual void RemoveDocument(gcpDocument* pDoc) = 0;
 	void SetMenu(const string& menuname, GtkWidget* menu) {Menus[menuname] = menu;}
 	GtkWidget* GetMenu(const string& name) {return Menus[name];}
 	gcpTool* GetActiveTool() {return m_pActiveTool;}
 	gcpDocument* GetActiveDocument() {return m_pActiveDoc;}
+	void SetActiveDocument (gcpDocument* pDoc) {m_pActiveDoc = pDoc;}
 	gcpTool* GetTool(const string& name) {return Tools[name];}
 	void SetTool (const string& toolname, gcpTool* tool) {Tools[toolname] = tool;}
 	GtkWidget* GetToolItem(const string& name) {return ToolItems[name];}
@@ -64,7 +71,7 @@ public:
 	void SetCurZ(int Z) {m_CurZ = Z;}
 	int GetCurZ() {return m_CurZ;}
 	void OnSaveAs();
-	bool FileProcess(const gchar* filename, int filetype, bool bSave, GtkWindow *window, gcpDocument *pDoc = NULL);
+	bool FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc = NULL);
 	void SaveWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
 	void OpenWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
 	void SaveGcp(const gchar* filename, gcpDocument* pDoc);
@@ -74,31 +81,41 @@ public:
 	xmlDocPtr GetXmlDoc() {return XmlDoc;}
 	void SetCallback(const string& name, GCallback cb) {Callbacks[name] = cb;}
 	GCallback GetCallback(const string& name) {return Callbacks[name];}
-//	void OnHelp (char* tag = NULL);
-	void OnBug ();
-	void OnWeb ();
-	void OnMail ();
 	void OnShowElements ();
 	void OnSaveAsImage ();
 	bool HaveGhemical () {return m_Have_Ghemical;}
 	bool HaveInChI () {return m_Have_InChI;}
-	char const* GetCurDir () {return CurDir;}
-	void SetCurDir (char const* dir);
 	int GetDocsNumber () {return m_Docs.size ();}
-	void ShowURI (string& uri);
 	void Zoom (double zoom);
+	void AddActions (GtkRadioActionEntry const *entries, int nb, char const *ui_description, gcpIconDesc const *icons);
+	void RegisterToolbar (char const *name, int index);
+	void OnToolChanged (GtkAction *current);
+	void AddWindow (gcpWindow *window);
+	void DeleteWindow (gcpWindow *window);
+	void NotifyIconification (bool iconified);
+	void NotifyFocus (bool has_focus, gcpWindow *window = NULL);
+	void CheckFocus ();
+	void CloseAll ();
+	list<char const*> &GetSupportedMimeTypes () {return m_SupportedMimeTypes;}
+
+	// virtual menus actions:
+	virtual void OnFileNew () = 0;
+	virtual void OnFileClose () = 0;
 
 protected:
-	virtual void AddDocument(gcpDocument* pDoc) = 0;
+//	virtual void AddDocument(gcpDocument* pDoc) = 0;
 	void InitTools();
+	void BuildTools ();
+	void ShowTools (bool visible);
 
 private:
 	void MergeNodes(BonoboUINode* Dest, BonoboUINode* Source);
 
 protected:
 	int m_CurZ;
-	gcpDocument* m_pActiveDoc;
-	std::list<gcpDocument*> m_Docs;
+	gcpDocument *m_pActiveDoc;
+	gcpWindow *m_pActiveWin;
+//	std::list<gcpDocument*> m_Docs;
 	map <string, GtkWidget*> Menus;
 	map <string, GtkWidget*> ToolItems;
 	map <string, GtkWidget*> Toolbars;
@@ -110,12 +127,21 @@ protected:
 	static map<string, string> m_SupportedPixbufFormats;
 	BonoboUINode* m_pNode;
 	char *FontName;
-//	string HelpBrowser;
-//	string HelpFilename;
-	string WebBrowser;
-	string MailAgent;
 	xmlDocPtr XmlDoc;
-	char *CurDir;
+	unsigned m_NumWindow; //used for new files (Untitled%d)
+
+private:
+	GtkIconFactory *IconFactory;
+	list<char const*> UiDescs;
+	GtkRadioActionEntry* RadioActions;
+	int m_entries;
+	map<int, string> ToolbarNames;
+	unsigned m_NumDoc; //used to build the name of the action associated with the menu
+	std::set<gcpWindow*> m_Windows;
+	int visible_windows;
+	bool focused;
+	list<char const*> m_SupportedMimeTypes;
+	unsigned m_idle;
 };
 
 extern list<gcpApplication*> Apps;
