@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,11 +30,11 @@
 #include "application.h"
 #include "view.h"
 #include "text.h"
+#include "theme.h"
 #include "settings.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
-#include "libgcpcanvas/gcp-canvas-text.h"
 #include <gcu/element.h>
 #include <math.h>
 
@@ -284,6 +284,7 @@ bool gcpFragment::OnEndUserAction(GtkTextBuffer *textbuffer)
 void gcpFragment::Add(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	gcpView* pView = pData->View;
 	if (m_ascent <= 0)
 	{
@@ -335,10 +336,10 @@ void gcpFragment::Add(GtkWidget* w)
 	GnomeCanvasItem* item = gnome_canvas_item_new(
 						group,
 						gnome_canvas_rect_ext_get_type(),
-						"x1", m_x * pData->ZoomFactor - pData->Padding - m_lbearing,
-						"y1", m_y * pData->ZoomFactor - pData->Padding - m_ascent + m_CHeight,
-						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding - m_lbearing,
-						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent + m_CHeight,
+						"x1", m_x * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_lbearing,
+						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent + m_CHeight,
+						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding () - m_lbearing,
+						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent + m_CHeight,
 						"fill_color", "white",
 						"outline_color", "white",
 						NULL);
@@ -349,8 +350,8 @@ void gcpFragment::Add(GtkWidget* w)
 						group,
 						gnome_canvas_rich_text_gcp_get_type(),
 						"text", "",
-						"x", m_x * pData->ZoomFactor - m_lbearing,
-						"y", m_y * pData->ZoomFactor - m_ascent + m_CHeight,
+						"x", m_x * pTheme->GetZoomFactor () - m_lbearing,
+						"y", m_y * pTheme->GetZoomFactor () - m_ascent + m_CHeight,
 						"width", m_length,
 						"height", m_height,
 						"grow_height", false,
@@ -372,29 +373,29 @@ void gcpFragment::Add(GtkWidget* w)
 			x = m_x + Dist * cos (Angle);
 			y = m_y - Dist * sin (Angle);
 		}
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 		switch (align) {
 		case -2:
-			x -= ChargeSignSize / 2.;
-			y += ChargeSignHeight / 2.;
+			x -= pTheme->GetChargeSignSize () / 2.;
+			y += pTheme->GetChargeSignHeight () / 2.;
 			break;
 		case -1:
-			x -= ChargeSignSize + 2.;
+			x -= pTheme->GetChargeSignSize () + 2.;
 			break;
 		case 0:
 		case -3:
-			x -= ChargeSignSize / 2.;
+			x -= pTheme->GetChargeSignSize () / 2.;
 			break;
 		case 1:
 			x += 2.;
 			break;
 		case 2:
-			x -= ChargeSignSize / 2.;
-			y -= ChargeSignHeight / 2.;
+			x -= pTheme->GetChargeSignSize () / 2.;
+			y -= pTheme->GetChargeSignHeight () / 2.;
 			break;
 		}
-		y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+		y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -405,8 +406,8 @@ void gcpFragment::Add(GtkWidget* w)
 					gnome_canvas_ellipse_ext_get_type (),
 					"x1", x,
 					"y1", y,
-					"x2", x + ChargeSignSize,
-					"y2", y + ChargeSignSize,
+					"x2", x + pTheme->GetChargeSignSize (),
+					"y2", y + pTheme->GetChargeSignSize (),
 					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 					"width_units", 0.5,
 					NULL
@@ -416,14 +417,14 @@ void gcpFragment::Add(GtkWidget* w)
 		path[0].code = ART_MOVETO_OPEN;
 		path[0].x3 = x + 1.;
 		path[1].code = ART_LINETO;
-		path[1].x3 = x + ChargeSignSize - 1.;
-		path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+		path[1].x3 = x + pTheme->GetChargeSignSize () - 1.;
+		path[0].y3 = path[1].y3 = y + pTheme->GetChargeSignSize () / 2.;
 		if (charge > 0) {
 			path[2].code = ART_MOVETO_OPEN;
 			path[2].y3 = y + 1.;
 			path[3].code = ART_LINETO;
-			path[3].y3 = y + ChargeSignSize - 1.;
-			path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+			path[3].y3 = y + pTheme->GetChargeSignSize () - 1.;
+			path[2].x3 = path[3].x3 = x + pTheme->GetChargeSignSize () / 2.;
 			path[4].code = ART_END;
 		} else
 			path[2].code = ART_END;
@@ -449,13 +450,24 @@ void gcpFragment::SetSelected(GtkWidget* w, int state)
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar *chargecolor, *color;
-	switch (state)
-	{	
-		case SelStateUnselected: color = "white"; chargecolor = "black"; break;
-		case SelStateSelected: chargecolor = color = SelectColor; break;
-		case SelStateUpdating: chargecolor = color = AddColor; break;
-		case SelStateErasing: chargecolor = color = DeleteColor; break;
-		default: color = "white"; chargecolor = "black"; break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "white";
+		chargecolor = "black";
+		break;
+	case SelStateSelected:
+		chargecolor = color = SelectColor;
+		break;
+	case SelStateUpdating:
+		chargecolor = color = AddColor;
+		break;
+	case SelStateErasing:
+		chargecolor = color = DeleteColor;
+		break;
+	default:
+		color = "white";
+		chargecolor = "black";
+		break;
 	}
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
@@ -469,18 +481,19 @@ void gcpFragment::SetSelected(GtkWidget* w, int state)
 void gcpFragment::Update(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasGroup *group = pData->Items[this];
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "fragment")),
-						"x", m_x * pData->ZoomFactor - m_lbearing,
-						"y", m_y * pData->ZoomFactor - m_ascent + m_CHeight,
+						"x", m_x * pTheme->GetZoomFactor () - m_lbearing,
+						"y", m_y * pTheme->GetZoomFactor () - m_ascent + m_CHeight,
 						"width", m_length,
 						"height", m_height,
 						NULL);
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
-						"x1", m_x * pData->ZoomFactor - pData->Padding - m_lbearing,
-						"y1", m_y * pData->ZoomFactor - pData->Padding - m_ascent + m_CHeight,
-						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding - m_lbearing,
-						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent + m_CHeight,
+						"x1", m_x * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_lbearing,
+						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent + m_CHeight,
+						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding () - m_lbearing,
+						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent + m_CHeight,
 						NULL);
 	void* item = g_object_get_data(G_OBJECT(group), "charge");
 	int charge = m_Atom->GetCharge ();
@@ -493,49 +506,49 @@ void gcpFragment::Update(GtkWidget* w)
 				x = m_x + Dist * cos (Angle);
 				y = m_y - Dist * sin (Angle);
 			}
-			x *= pData->ZoomFactor;
-			y *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
+			y *= pTheme->GetZoomFactor ();
 			switch (align) {
 			case -2:
-				x -= ChargeSignSize / 2.;
-				y += ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
+				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				x -= ChargeSignSize + 2.;
+				x -= pTheme->GetChargeSignSize () + 2.;
 				break;
 			case 0:
 			case -3:
-				x -= ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			case 1:
 				x += 2.;
 				break;
 			case 2:
-				x -= ChargeSignSize / 2.;
-				y -= ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
+				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
-			y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+			y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
 			item = g_object_get_data (G_OBJECT (group), "circle");
 			g_object_set (G_OBJECT (item),
 						"x1", x,
 						"y1", y,
-						"x2", x + ChargeSignSize,
-						"y2", y + ChargeSignSize,
+						"x2", x + pTheme->GetChargeSignSize (),
+						"y2", y + pTheme->GetChargeSignSize (),
 						NULL);
 			item = g_object_get_data (G_OBJECT (group), "sign");
 			ArtBpath *path = art_new (ArtBpath, 5);
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = x + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = x + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+			path[1].x3 = x + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = y + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = y + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = y + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+				path[3].y3 = y + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = x + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -548,33 +561,33 @@ void gcpFragment::Update(GtkWidget* w)
 		} else {
 			GnomeCanvasGroup *chgp;
 			int align = GetChargePosition(m_Atom, Pos, Angle, x, y);
-			x *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
 			if (Dist != 0.) {
 				x = m_x + Dist * cos (Angle);
 				y = m_y - Dist * sin (Angle);
 			}
-			y *= pData->ZoomFactor;
+			y *= pTheme->GetZoomFactor ();
 			switch (align) {
 			case -2:
-				x -= ChargeSignSize / 2.;
-				y += ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
+				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				x -= ChargeSignSize + 2.;
+				x -= pTheme->GetChargeSignSize () + 2.;
 				break;
 			case 0:
 			case -3:
-				x -= ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			case 1:
 				x += 2.;
 				break;
 			case 2:
-				x -= ChargeSignSize / 2.;
-				y -= ChargeSignSize / 2.;
+				x -= pTheme->GetChargeSignSize () / 2.;
+				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
-			y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+			y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
 			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 						group,
 						gnome_canvas_group_ext_get_type(),
@@ -585,8 +598,8 @@ void gcpFragment::Update(GtkWidget* w)
 						gnome_canvas_ellipse_ext_get_type (),
 						"x1", x,
 						"y1", y,
-						"x2", x + ChargeSignSize,
-						"y2", y + ChargeSignSize,
+						"x2", x + pTheme->GetChargeSignSize (),
+						"y2", y + pTheme->GetChargeSignSize (),
 						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 						"width_units", 0.5,
 						NULL
@@ -596,14 +609,14 @@ void gcpFragment::Update(GtkWidget* w)
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = x + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = x + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+			path[1].x3 = x + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = y + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = y + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = y + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+				path[3].y3 = y + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = x + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -1010,9 +1023,10 @@ void gcpFragment::AnalContent(GtkTextIter& start, GtkTextIter& end)
 Must return NULL if active tool is FragmentTool because this tool needs a fragment, not an atom
 TODO: use x and y to figure the best atom in the fragment
 */
-Object* gcpFragment::GetAtomAt(double x, double y, double z)
+Object* gcpFragment::GetAtomAt (double x, double y, double z)
 {
-	gcpDocument* pDoc = (gcpDocument*)GetDocument();
+	gcpDocument* pDoc = (gcpDocument*) GetDocument ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpApplication* pApp = pDoc->GetApplication();
 	if (pApp->GetActiveTool() == pApp->GetTool("Fragment")) return NULL;
 	if (m_Atom->GetBondsNumber () || m_Atom->GetCharge ())
@@ -1021,8 +1035,8 @@ Object* gcpFragment::GetAtomAt(double x, double y, double z)
 	GtkWidget* pWidget = pDoc->GetView()->GetWidget();
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
 	double x0, y0;
-	x0 = (x - m_x) * pData->ZoomFactor + m_lbearing;
-	y0 = (y - m_y) * pData->ZoomFactor + m_ascent;
+	x0 = (x - m_x) * pTheme->GetZoomFactor () + m_lbearing;
+	y0 = (y - m_y) * pTheme->GetZoomFactor () + m_ascent;
 	if ((x0 < 0.) || (x0 > m_length) || (y0 < 0.) || (y0 > m_height)) return NULL;
 	GnomeCanvasGroup *item = pData->Items[this];
 	if (!item) return NULL;
@@ -1043,14 +1057,14 @@ Object* gcpFragment::GetAtomAt(double x, double y, double z)
 	m_BeginAtom = gtk_text_buffer_create_mark(m_buf, "ab0", &start, false);
 	gtk_text_buffer_delete_mark(m_buf, m_EndAtom);
 	m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &end, true);
-	m_x -= m_lbearing / pData->ZoomFactor ;
+	m_x -= m_lbearing / pTheme->GetZoomFactor () ;
 	GdkRectangle rect;
 	gnome_canvas_rich_text_ext_get_iter_location(text, &start, &rect);
 	m_lbearing = rect.x;
 	gnome_canvas_rich_text_ext_get_iter_location(text, &end, &rect);
 	m_lbearing += rect.x;
 	m_lbearing /=  2;
-	m_x += m_lbearing / pData->ZoomFactor;
+	m_x += m_lbearing / pTheme->GetZoomFactor ();
 	m_Atom->SetCoords(m_x, m_y);
 	return m_Atom;
 }
@@ -1094,7 +1108,8 @@ int gcpFragment::GetElementAtIter(GtkTextIter* start, GtkTextIter* end)
 	gtk_text_buffer_get_iter_at_offset(m_buf, end, offset + 3);
 	while (gtk_text_iter_compare(start, end))
 	{
-		if ((Z = Element::Z(gtk_text_buffer_get_text(m_buf, start, end, false)))) return Z;
+		if ((Z = Element::Z (gtk_text_buffer_get_text (m_buf, start, end, false))))
+			return Z;
 		gtk_text_iter_backward_char(end);
 	}
 	return 0;
@@ -1106,6 +1121,7 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, unsigned char& Pos, d
 	GtkTextIter iter;
 	double width, height;
 	gcpDocument* pDoc = (gcpDocument*)GetDocument();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	if (!pDoc) return 0;
 	GtkWidget* pWidget = pDoc->GetView()->GetWidget();
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
@@ -1127,8 +1143,8 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, unsigned char& Pos, d
 	gtk_text_buffer_get_iter_at_mark(m_buf, &iter, m_EndAtom);
 	gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
 	width += rect.x;
-	width /= pData->ZoomFactor;
-	height = m_height / pData->ZoomFactor;
+	width /= pTheme->GetZoomFactor ();
+	height = m_height / pTheme->GetZoomFactor ();
 	if (!gtk_text_iter_is_end(&iter)) result &= 0xb6;
 	if (m_Atom->GetBondsNumber())
 	{
