@@ -4,7 +4,7 @@
  * GChemPaint atoms plugin
  * elementtool.cc 
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,6 +29,7 @@
 #include "lib/settings.h"
 #include "lib/application.h"
 #include "lib/mendeleiev.h"
+#include "lib/theme.h"
 #include <math.h>
 #include <string.h>
 #include "libgcpcanvas/gcp-canvas-group.h"
@@ -57,10 +58,11 @@ bool gcpElementTool::OnClicked()
 	PangoLayout* pl = pango_layout_new(m_pView->GetPangoContext());
 	pango_layout_set_text(pl, symbol, strlen(symbol));
 	gint width = pango_layout_get_width(pl);
-	m_x1 = m_x0 - (double)width / 2 - m_pData->Padding;
-	m_y1 = m_y0 - m_pView->GetFontHeight() / 2 - m_pData->Padding;
-	m_x2 = m_x0 + (double)width / 2 + m_pData->Padding;
-	m_y2 = m_y0 + m_pView->GetFontHeight() / 2 + m_pData->Padding;
+	double padding = m_pView->GetDoc ()->GetTheme ()->GetPadding ();
+	m_x1 = m_x0 - (double)width / 2 - padding;
+	m_y1 = m_y0 - m_pView->GetFontHeight() / 2 - padding;
+	m_x2 = m_x0 + (double)width / 2 + padding;
+	m_y2 = m_y0 + m_pView->GetFontHeight() / 2 + padding;
 	m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 	gnome_canvas_item_new(
 					(GnomeCanvasGroup*)m_pItem,
