@@ -32,26 +32,32 @@ extern "C" {
 	void gnome_authentication_manager_init ();
 }
 
-gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
+#warning "the following lines should be removed for stable releases"
+#undef PACKAGE
+#define PACKAGE "gchempaint-unstable" 
+
+void cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
 {
 	char *version = g_strconcat (_("GChemPaint version: "), VERSION, NULL);
 	puts (version);
 	g_free (version);
 	exit (0);
-	return TRUE;
 }
 
+extern bool new_ui;
+
 static GOptionEntry entries[] = 
 {
-  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemPaint version", NULL },
-   { NULL }
+  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "Prints GChemPaint version", NULL },
+  { "new-ui", 'n', 0, G_OPTION_ARG_NONE, (void*) &new_ui, "Use new user interface", NULL },
+  { NULL }
 };
 
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError *error = NULL;
-	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
+	bindtextdomain(GETTEXT_PACKAGE, DATADIR"/locale");
 #ifdef ENABLE_NLS
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
@@ -64,16 +70,16 @@ int main(int argc, char *argv[])
 		context = g_option_context_new (_(" [file...]"));
 		g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 		g_option_context_add_group (context, gtk_get_option_group (TRUE));
+		g_option_context_set_help_enabled (context, TRUE);
 		g_option_context_parse (context, &argc, &argv, &error);
 		if (error) {
 			puts (error->message);
 			g_error_free (error);
 			return -1;
 		}
-	} else {
-		argc --;
-		argv ++;
 	}
+	argv ++;
+	argc --;
 
 	gcpStandaloneApp* App = new gcpStandaloneApp();
 	GnomeVFSURI *uri, *auri;
@@ -103,7 +109,8 @@ int main(int argc, char *argv[])
 	if (App->GetDocsNumber () == 0)
 		App->OnFileNew();
 
-	App->Show ();
+	if (!new_ui)
+		App->Show ();
 
 	gtk_main();
 	delete App;
