@@ -40,7 +40,6 @@ public:
 	virtual void ClearStatus();
 	virtual void SetStatusText(const char* text);
 	virtual GtkWindow* GetWindow();
-	virtual void RemoveDocument(gcpDocument* pDoc);
 	void OnFileNew();
 	void OnSave();
 	void OnFileOpen();
@@ -64,8 +63,7 @@ public:
 	void OnSaveAll ();
 	void Show ();
 
-protected:
-	virtual void AddDocument(gcpDocument* pDoc);
+	void NoMoreDocsEvent ();
 
 private:
 	GtkWidget* CreateView();
